describe('Unit: Integrated Data', function() {

  let subject, cdmHelpersMock;

  beforeEach(function() {
    angular.mock.module('app');

    cdmHelpersMock = jasmine.createSpyObj('cdmHelpersMock', ['parseDataModel', 'getDatumVal'])

    angular.mock.module(($provide) => {
      $provide.value('CdmHelpers', cdmHelpersMock);
    });
    
    angular.mock.inject((IntegratedData) => {
      subject = IntegratedData;
    });
  });

  it('should exist', function() {
    expect(subject).toBeDefined();
  });

  it('should retrieve the matter entity on request', function() {
    var payload = {
      MatterEntity: 'some matter entity'
    };

    subject.register(payload);

    expect(subject.retrieveMatterEntity()).toBe(payload.MatterEntity);
  });

  it('should retrieve related entities on request', function() {
    var payload = 'totally a payload';
    var types = 'type parameter';
    var relationship = 'relationship parameter';

    subject.register(payload);
    subject.retrieveRelatedEntities(types, relationship);

    expect(cdmHelpersMock.parseDataModel).toHaveBeenCalledWith(types, relationship, null, payload);
  });

  it('should retrieve datum values from a given model on request', function() {
    var payload = {
      MatterEntity: 'some matter entity'
    };
    var model = {
      OtherData: ['otherdata1', 'otherdata2']
    }
    var key = 'some key';

    cdmHelpersMock.getDatumVal.and.callFake(function() {
      return 'test return from getdatumval';
    });

    subject.register(payload);
    var result = subject.retrieveDatumByKey(key, model);

    expect(cdmHelpersMock.getDatumVal).toHaveBeenCalledWith(model.OtherData, key);
    expect(result).toBe('test return from getdatumval');
  });

  it('should retrieve datum values from the payload on request', function() {
    var payload = {
      MatterEntity: {
        OtherData: ['otherdata ichi', 'otherdata ni']
      }
    };
    var key = 'some other key';

    cdmHelpersMock.getDatumVal.and.callFake(function() {
      return 'test other return from getdatumval';
    });

    subject.register(payload);
    var result = subject.retrieveDatumByKey(key);

    expect(cdmHelpersMock.getDatumVal).toHaveBeenCalledWith(payload.MatterEntity.OtherData, key);
    expect(result).toBe('test other return from getdatumval');
  });

  it('should return empty string when retrieving datum values from invalid payload', function() {
    var key = 'some other key';

    cdmHelpersMock.getDatumVal.and.callFake(function() {
      return 'test some return from getdatumval';
    });

    //no matter entity
    subject.register({
      OtherData: ['something']
    });
    var result = subject.retrieveDatumByKey(key);

    expect(cdmHelpersMock.getDatumVal).not.toHaveBeenCalled();
    expect(result).toBe('');

    //empty array
    subject.register({
      MatterEntity: {OtherData: []},
    });
    var result = subject.retrieveDatumByKey(key);

    expect(cdmHelpersMock.getDatumVal).not.toHaveBeenCalled();
    expect(result).toBe('');

    //object
    subject.register({
      MatterEntity: {OtherData: {}},
    });
    var result = subject.retrieveDatumByKey(key);

    expect(cdmHelpersMock.getDatumVal).not.toHaveBeenCalled();
    expect(result).toBe('');

    //string
    subject.register({
      MatterEntity: {OtherData: ''},
    });
    var result = subject.retrieveDatumByKey(key);

    expect(cdmHelpersMock.getDatumVal).not.toHaveBeenCalled();
    expect(result).toBe('');
  });

  it('should return empty string when retrieving datum values from invalid model', function() {
    var key = 'some other key';
    var payload = {
      MatterEntity: 'some matter entity'
    };

    cdmHelpersMock.getDatumVal.and.callFake(function() {
      return 'test some return from getdatumval';
    });
    subject.register(payload);

    //empty array
    var result = subject.retrieveDatumByKey(key, {OtherData: []});
    expect(cdmHelpersMock.getDatumVal).not.toHaveBeenCalled();
    expect(result).toBe('');

    //object
    var result = subject.retrieveDatumByKey(key, {OtherData: {}});
    expect(cdmHelpersMock.getDatumVal).not.toHaveBeenCalled();
    expect(result).toBe('');

    //string
    var result = subject.retrieveDatumByKey(key, {OtherData: ''});
    expect(cdmHelpersMock.getDatumVal).not.toHaveBeenCalled();
    expect(result).toBe('');
  });

  it('should clear on request', function() {
    var payload = {
      MatterEntity: 'some matter entity'
    };

    subject.register(payload);

    var result1 = subject.retrieveMatterEntity();
    expect(result1).toBe(payload.MatterEntity);

    subject.clear();
    result1 = subject.retrieveMatterEntity();
    expect(result1).toBeUndefined();
  });
});